"""Clean Image Crop Uploader (CICU) provides AJAX file upload and image CROP functionalities for ImageFields with a simple widget replacement in the form."""

VERSION = (0, 2, 2)

__version__ = '.'.join(map(str, VERSION))



from django.db import models
from django.utils.translation import ugettext_lazy as _


class UploadedFile(models.Model):
    creation_date = models.DateTimeField(_('creation date'), auto_now_add=True)
    file = models.FileField(_('file'), upload_to='ajax_uploads/')

    class Meta:
        ordering = ('id',)
        verbose_name = _('uploaded file')
        verbose_name_plural = _('uploaded files')

    def __unicode__(self):
        return unicode(self.file)

    def delete(self, *args, **kwargs):
        super(UploadedFile, self).delete(*args, **kwargs)
        if self.file:
            self.file.delete()
    delete.alters_data = True



from django.conf import settings


# Number of seconds to keep uploaded files. The clean_uploaded command will
# delete them after this has expired.
UPLOADER_DELETE_AFTER = getattr(settings, 'UPLOADER_DELETE_AFTER', 60 * 60)
IMAGE_CROPPED_UPLOAD_TO = getattr(settings, 'IMAGE_CROPPED_UPLOAD_TO', 'cicu_croped/')



from django import forms
from django.conf import settings
from django.core.files import File
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from PIL import Image
from .models import UploadedFile


class CicuException(Exception):
    pass


class CicuUploderInput(forms.ClearableFileInput):
    template_with_clear = ''  # We don't need this
    template_with_initial = '%(input)s'
    #basic configuration for jcrop from django
    optionsInput = '<input id="cicu-options" data-size-warning="%s"  data-ratio-width="%s" data-ratio-height="%s" data-modal-button-label="%s" data-change-button-text="%s" data-size-alert-message="%s" data-size-error-message="%s" data-modal-save-crop-message="%s" data-modal-close-crop-message="%s" data-uploading-message="%s" data-file-upload-label="%s" style="display: none;" />'

    def __init__(self, attrs=None, options=None):
        if not options: options = {}
        super(CicuUploderInput ,self).__init__(attrs)

        #jcrop configuration
        self.options = ()
        self.options += (options.get('sizeWarning','True'),)
        self.options += (options.get('ratioWidth', ''),)
        self.options += (options.get('ratioHeight', ''),)
        #input message customization and translation
        self.options += (options.get('modalButtonLabel',_('Upload image') ),)
        self.options += (options.get('changeButtonText',_('Change Image') ),)
        self.options += (options.get('sizeAlertMessage',_('Warning: the area selected is little, min size:') ),)
        self.options += (options.get('sizeErrorMessage',_("Image don't have minimal size ") ),)
        self.options += (options.get('modalSaveCropMessage',_('Set image') ),)
        self.options += (options.get('modalCloseCropMessage',_('Close') ),)
        self.options += (options.get('uploadingMessage',_('Uploading your image') ),)
        self.options += (options.get('fileUploadLabel',_('Select image from your computer') ),)

    def render(self, name, value, attrs=None):
        attrs = attrs or {}
        if value:
            filename = u'%s%s' % (settings.MEDIA_URL, value)
        else:
            filename = ''
        attrs.update({
            'class': attrs.get('class', '') + 'ajax-upload',
            'data-filename': filename,  # This is so the javascript can get the actual value
            'data-required': self.is_required or '',
            'data-upload-url': reverse('ajax-upload'),
            'data-crop-url': reverse('cicu-crop'),
            'type': 'file',
            'accept' : 'image/*',
        })
        output = super(CicuUploderInput, self).render(name, value, attrs)
        option = self.optionsInput % self.options
        autoDiscoverScript = "<script>$(function(){CicuWidget.autoDiscover();});</script>"
        return mark_safe(output+option+autoDiscoverScript)

    def value_from_datadict(self, data, files, name):
        # If a file was uploaded or the clear checkbox was checked, use that.
        file = super(CicuUploderInput, self).value_from_datadict(data, files, name)
        if file is not None:  # super class may return a file object, False, or None
            return file  # Default behaviour
        elif name in data:  # This means a file id was specified in the POST field
            try:
                uploaded_file = UploadedFile.objects.get(id=data.get('image'))
                img = Image.open(uploaded_file.file.path, mode='r')
                width, height = img.size
                if (width < self.options[1] or height < self.options[2]) and self.options[0] == 'True':
                    raise Exception('Image don\'t have correct ratio %sx%s'  % (self.options[1], self.options[2]))
                return uploaded_file.file
            except Exception, e:
                return None
        return None

    class Media:
        js = (
            "cicu/js/jquery.Jcrop.min.js",
            "cicu/js/jquery.iframe-transport.js",
            "cicu/js/cicu-widget.js",
            )

        css = {'all': ("cicu/css/jquery.Jcrop.min.css","cicu/css/cicu-widget.css")}



from django.http import HttpResponse, HttpResponseBadRequest
from django.utils import simplejson
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.core.files import File
from PIL import Image
from os import path, sep, makedirs
from .forms import UploadedFileForm
from .models import UploadedFile
from .settings import IMAGE_CROPPED_UPLOAD_TO
from django.conf import settings


@csrf_exempt
@require_POST
def upload(request):
    form = UploadedFileForm(data=request.POST, files=request.FILES)
    if form.is_valid():
        uploaded_file = form.save()
        # pick an image file you have in the working directory
        # (or give full path name)
        img = Image.open(uploaded_file.file.path, mode='r')
        # get the image's width and height in pixels
        width, height = img.size
        data = {
            'path': uploaded_file.file.url,
            'id' : uploaded_file.id,
            'width' : width,
            'height' : height,
        }
        return HttpResponse(simplejson.dumps(data))
    else:
        return HttpResponseBadRequest(simplejson.dumps({'errors': form.errors}))

@csrf_exempt
@require_POST
def crop(request):
    try:
        if request.method == 'POST':
            box = request.POST.get('cropping', None)
            imageId = request.POST.get('id', None)
            uploaded_file = UploadedFile.objects.get(id=imageId)
            img = Image.open( uploaded_file.file.path, mode='r' )
            values = [int(x) for x in box.split(',')]

            width = abs(values[2] - values[0])
            height = abs(values[3] - values[1])
            if width and height and (width != img.size[0] or height != img.size[1]):
                croppedImage = img.crop(values).resize((width,height),Image.ANTIALIAS)

            else:
                raise

            pathToFile = path.join(settings.MEDIA_ROOT,IMAGE_CROPPED_UPLOAD_TO)
            if not path.exists(pathToFile):
                makedirs(pathToFile)
            pathToFile = path.join(pathToFile,uploaded_file.file.path.split(sep)[-1])
            croppedImage.save(pathToFile)

            new_file = UploadedFile()
            f = open(pathToFile, mode='r')
            new_file.file.save(uploaded_file.file.name, File(f))
            f.close()

            data = {
                'path': new_file.file.url,
                'id' : new_file.id,
            }

            return HttpResponse(simplejson.dumps(data))

    except Exception, e:
        return HttpResponseBadRequest(simplejson.dumps({'errors': 'illegal request'}))



import uuid

from django import forms

from .models import UploadedFile

class UploadedFileForm(forms.ModelForm):

    class Meta:
        model = UploadedFile
        fields = ('file',)

    def clean_file(self):
        data = self.cleaned_data['file']
        # Change the name of the file to something unguessable
        # Construct the new name as <unique-hex>-<original>.<ext>
        data.name = u'%s-%s' % (uuid.uuid4().hex, data.name)
        return data



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'UploadedFile'
        db.create_table('cicu_uploadedfile', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('creation_date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('file', self.gf('django.db.models.fields.files.FileField')(max_length=100)),
        ))
        db.send_create_signal('cicu', ['UploadedFile'])


    def backwards(self, orm):
        # Deleting model 'UploadedFile'
        db.delete_table('cicu_uploadedfile')


    models = {
        'cicu.uploadedfile': {
            'Meta': {'ordering': "('id',)", 'object_name': 'UploadedFile'},
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['cicu']


__author__ = 'Alfredo Saglimbeni'

from distutils.core import setup
from setuptools import setup, find_packages

setup(name = "clean-image-crop-uploader",
    version = "0.2.2",
    description = "Clean Image Crop Uploader (CICU) provides AJAX file upload and image CROP functionalities for ImageFields with a simple widget replacement in the form. It use Modal from twitter-bootstrap.",
    long_description=open('README.rst').read(),
    author = "asagli",
    author_email = "alfredo.saglimbeni@gmail.com",
    url = "",
    packages = find_packages(),
    include_package_data=False,
    install_requires = [
        'PIL==1.1.7','django>=1.4.3','south>=0.7.6'
        ],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
)


from django.contrib import admin

from .models import UploadedFile


class UploadedFileAdmin(admin.ModelAdmin):
    list_display = ('__unicode__',)
    date_hierarchy = 'creation_date'
    search_fields = ('file',)


admin.site.register(UploadedFile, UploadedFileAdmin)
